#!/usr/bin/env python

import string


def parse(path):
    with open(path) as f:
        string = f.read()
    init()
    return parseObj({}, string, 0)


def list(path):
    with open(path) as f:
        str = f.read()
    str = str[7:]

    extra = "\"'\n :"
    for char in extra:
        str = str.replace(char, "")
    list = str.split(",")
    return list


def init():
    global word
    global lastWord
    global lastChar
    word = ""
    lastWord = ""
    lastChar = ""


def parseObj(root, raw, index):
    global word
    global lastWord
    global lastChar
    char = ""
    length = len(raw) - 1
    blocks = [root]
    tag = ""
    tags = [tag]
    obj = root
    i = index
    while i < length:
        i += 1
        lastChar = char
        char = raw[i]
        if char == "/" and lastChar == "/":
            i = skipToNext(raw, "\n", i)
            char = raw[i]

        if char == " " or char == "\n":
            keepWord()

        if "(" == char and tag == "repositories":
            obj[word] = ""
            keepWord()
            i = skipToNext(raw, ")", i)
            continue

        if "{" == char:
            keepWord()
            if tag == "dependencies":
                i = skipToNext(raw, "}", i)
                continue
            tags.append(tag)
            tag = lastWord

            if tag != "maven":
                blocks.append(obj)
                child = {}
                obj[tag] = child
                obj = child

        if "}" == char:
            if tag != "maven":
                obj = blocks.pop()
            tag = tags.pop()

        if tag in ["dependencies", "repositories", "maven"]:
            i = checkString(raw, char, "'", obj, i)
            i = checkString(raw, char, "\"", obj, i)
            if tag == "dependencies" and lastWord in ["group", "name", "version"]:
                mergeDependency(obj)

        if isText(char):
            word += char

    return root


def mergeDependency(obj):
    group = ""
    name = ""
    version = ""
    for key in obj:
        if obj[key] == "group":
            group = key
        elif obj[key] == "name":
            name = key
        elif obj[key] == "version":
            version = key
    if group and name and version:
        obj.pop(group, None)
        obj.pop(name, None)
        obj.pop(version, None)
        obj[group + ":" + name + ":" + version] = ""


def skipToNext(raw, target, index):
    length = len(raw)
    i = index
    while i < length and raw[i] != target:
        i += 1
    return i


def checkString(raw, char, target, obj, index):
    if target == char:
        end = skipToNext(raw, target, index + 1)
        obj[raw[index + 1:end]] = lastWord
        return end
    return index


def isText(char):
    if char in string.ascii_letters:
        return True
    elif char == ".":
        return True
    return False


def keepWord():
    global word
    global lastWord
    if word:
        lastWord = word
        word = ""



#!/usr/bin/python

import os
import requests


def download(url, path):
    r = requests.get(url, stream=True)
    if r.status_code == 404 or r.status_code == 401:
        return False
    if not os.path.exists(os.path.dirname(path)):
        try:
            os.makedirs(os.path.dirname(path))
        except OSError as exc:
            return False
    elif os.path.exists(path):
        return True
    handle = open(path, "wb")
    for chunk in r.iter_content(chunk_size=512):
        if chunk:  # filter out keep-alive new chunks
            handle.write(chunk)

    print("download: " + url)
    return True



#!/usr/bin/env python

import syndle.gradle as gradle
import syndle.remote as remote
import os
import timeit
import sys


__version__ = "0.1"

BUILDSCRIPT = "buildscript"
REPOSITORIES = "repositories"
ALLPROJECTS = "allprojects"
DEPENDENCIES = "dependencies"


def parse(path):
    start_time = timeit.default_timer()
    parseInner(path)
    elapsed = timeit.default_timer() - start_time
    print("total: " + str(elapsed))


def clone(package, servers):
    start_time = timeit.default_timer()
    remote.parse([package], servers)
    elapsed = timeit.default_timer() - start_time
    print("total: " + str(elapsed))


def parseInner(path):
    if not path.endswith("/"):
        path += "/"
    if not os.path.exists(path):
        print("not exist path")
        return

    projects = gradle.list(path + "settings.gradle")
    root = gradle.parse(path + "build.gradle")

    if BUILDSCRIPT in root:
        script = root[BUILDSCRIPT]
        print(script)
        servers = findValue(REPOSITORIES, script)
        deps = findValue(DEPENDENCIES, script)
        if deps:
            remote.parse(deps, servers)

    if ALLPROJECTS in root:
        rootServers = findValue(REPOSITORIES, root[ALLPROJECTS])

    for p in projects:
        child = gradle.parse(path + "/" + p + "/build.gradle")
        servers = findValue(REPOSITORIES, child)
        deps = findValue(DEPENDENCIES, child)
        if deps:
            remote.parse(deps, rootServers + servers)


def findValue(key, obj):
    if key in obj:
        if (sys.version_info > (3, 0)):
            return list(obj[key])
        else:
            return obj[key].keys()
    return []



#!/usr/bin/env python

import syndle.service as service
import os
from xml.dom import minidom

PATH_MAVEN = os.path.expanduser("~/.m2/repository/")
URL_JCENTER = "https://jcenter.bintray.com/"
EXT_LIST = [
    ".pom",
    ".jar",
    ".aar",
    # "-javadoc.jar",
    # "-sources.jar",
]

AAR_LIST = [
    ".pom",
    ".aar"
]

HREF_LENGTH = len("href=\":")


def parse(packages, urls=[URL_JCENTER], tree=[]):
    global servers
    servers = formServer(urls)
    global history
    history = tree
    for package in packages:
        data = package.split(":")
        if len(data) != 3:
            return
        group = data[0]
        name = data[1]
        version = data[2]
        if version.endswith("@aar"):
            version = version.replace("@aar", "")
            dispatch(group, name, version, True)
        else:
            dispatch(group, name, version)


def dispatch(group, name, version, aar=False):
    global history
    if not version:
        print("can't parse version: " + group + "." + name)
        return
    path = formPath(group, name)
    obj = path + version + "/"
    if obj in history:
        return False
    pom = PATH_MAVEN + obj + name + "-" + version + ".pom"
    if os.path.exists(pom):
        print("exist: " + pom)
        findDependency(pom, version)
        return False
    for url in servers:
        if maven(url, path, name, version, aar):
            break
    history.append(obj)


def maven(url, obj, name, version, aar):
    count = 0
    if aar:
        list = AAR_LIST
    else:
        list = EXT_LIST
    if not download(url, obj + "maven-metadata.xml", version):
        return False
    obj += version + "/"
    title = name + "-" + version
    for ext in list:
        try:
            if download(url, obj + title + ext, version):
                findDependency(PATH_MAVEN + obj + title + ext, version)
                count += 1
            elif ext == ".pom":
                return False
            if count == 2:
                return True
        except Exception as e:
            print(e)
    return False


def download(server, path, version):
    if service.download(server + path, PATH_MAVEN + path):
        return True
    return False


def findDependency(path, rootVersion):
    if path.endswith(".pom"):
        DOMTree = minidom.parse(path)

        root = DOMTree.documentElement
        try:
            dependencies = root.getElementsByTagName("dependencies")[0]
            for dependency in dependencies.getElementsByTagName("dependency"):
                try:
                    scope = xmlValue(dependency, "scope")
                    if scope == "test":
                        continue
                except Exception:
                    pass
                group = xmlValue(dependency, "groupId")
                name = xmlValue(dependency, "artifactId")
                version = xmlValue(dependency, "version")
                if version:
                    if version.startswith("["):
                        version = version.replace("[", "").replace("]", "")
                    if version == "${project.version}":
                        version = rootVersion
                    if "${" in version:  # TODO not support currently
                        continue
                dispatch(group, name, version)
        except Exception as e:
            pass


def xmlValue(obj, name):
    return obj.getElementsByTagName(name)[0].childNodes[0].data


def formServer(urls):
    list = []
    for url in urls:
        if url == "jcenter":
            list.append(URL_JCENTER)
        elif url == "google":
            list.append("https://maven.google.com/")
        elif url.startswith("http") or url.startswith("https"):
            if not url.endswith("/"):
                url += "/"
            list.append(url)
    return list


def formPath(group, name):
    return group.replace(".", "/") + "/" + name + "/"



from setuptools import setup

setup(name='syndle',
      version='0.1',
      description='sync gradle dependency to local maven',
      long_description='andle is a command line tool to help you sync dependencies, sdk or build tool version in gradle base Android projects.',
	  keywords='android gradle config build version dependency sync',
	  scripts=['bin/syndle'],
      url='http://github.com/Jintin/syndle',
      author='jintin',
      author_email='jintinapps@gmail.com',
      test_suite='nose.collector',
	  tests_require=['nose'],
      license='MIT',
      packages=['syndle'],
      install_requires=["requests"],
      zip_safe=False)
